/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000, 2001 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#if !db_h
#define db_h 1

#include <stdarg.h>

#undef unused
#include <libpq-fe.h>
#define unused attribute ((unused))

typedef PGresult db_result;

/* Setup. */
void db_open (const char *dbname);
int db_open_p (void);
void db_close (void);

/* Transactions. */
void db_begin (void);
void db_end (void);

/* Queries. */

/* Query flags. */
enum
  {
    DB_FAIL_OK = 001,		/* Failure is okay. */
    DB_TUPLES = 002,		/* Expect tuples to be returned. */
    DB_FORMAT = 004		/* Format the query with vformat. */
  };
int db_query (const char *query, int flags, db_result **, ...)
     attribute ((format (printf, 1, 4)));
const char *db_error (void);

/* Query result functions. */
size_t dbr_nrows (const db_result *);
size_t dbr_ncols (const db_result *);
char *dbr_value (const db_result *, int row, int col);
size_t dbr_value_length (const db_result *, int row, int col);
int dbr_value_is_null (const db_result *, int row, int col);
void dbr_destroy (db_result *);

/* Commonly needed operations implemented in terms of above
   functions. */
struct context;
int get_seq (void);
int get_file_seq (struct context *, const char *filename);

/* Similiar in concept to printf(), specialized for use with SQL. */
size_t vformat (char *buffer, size_t size, const char *format, va_list args);

#endif /* db.h */
