/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000, 2001 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#if !identity_h
#define identity_h 1

#include <sys/types.h>

struct context
  {
    char usr[9];		/* Username. */
    char *fullusr;		/* User's full name. */
    uid_t uid;			/* User's UID. */
    char *home;			/* User's home directory name. */
    struct std *options;	/* User options. */

    char grp[9];		/* Workgroup name. */
    char *fullgrp;		/* Workgroup's full name. */
    gid_t gid;			/* Workgroup's GID. */
    char *shared;		/* Workgroup's shared directory name. */
    char *web;			/* Workgroup's web directory name. */
    struct workgroup *wg;	/* Workgroup user information. */
  };

struct workgroup
  {
    struct std *users;		/* Maps username to full name. */

    int n;			/* Number of users. */
    char **name;		/* Users' names. */
    char **fullname;		/* Users' full names. */
  };
    
void read_user_list (struct workgroup **, const char *grp);

const char *seq_no_to_resource_name (int seq);
const char *uid_to_username (uid_t);
const char *gid_to_groupname (gid_t);
const char *username_to_fullname (struct workgroup *, const char *username);
void init_context (struct context *);
void destroy_context (struct context *);
char *relevant_file_crit (struct context *);
void verify_file_owner (const char *filename, FILE *f, uid_t uid);

/* User information ADT. */

/* Information about one user.
   All dynamically allocated. */
struct user_info
  {
    char *usr;		/* Username. */
    char *fullname;	/* Full name. */
    int recordable;	/* Is the user's email recordable? */
    int locked;		/* Is the user's account locked? */
    char *last_usage;	/* Date and time of last usage as a string. */
    char *email;	/* Upstream email address. */
  };

/* Fields to fill. */
enum ui_fields
  {
    UI_USR = 001,
    UI_FULLNAME = 002,
    UI_RECORDABLE = 004,
    UI_LOCKED = 010,
    UI_LAST_USAGE = 020,
    UI_EMAIL = 040,
    UI_ALL = 0777
  };

struct id_list;
void get_user_info (const struct id_list *, struct user_info *,
			   enum ui_fields);
void free_user_info (struct user_info *, size_t);

#endif /* identity.h */
