/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#include <stdlib.h>
#include <stdio.h>
#include "postgres.h"
#include "utils/array.h"
#include "utils/memutils.h"
#include "utils/palloc.h"

int
_int4_depth (ArrayType *a) 
{
  const int al = ARR_DIMS(a)[0];

  return al;
}

int
_int4_lt (ArrayType *a, ArrayType *b)
{
  const int al = ARR_DIMS(a)[0];
  const int bl = ARR_DIMS(b)[0];
  const int *const ad = (int *) ARR_DATA_PTR(a);
  const int *const bd = (int *) ARR_DATA_PTR(b);
  int i;

  /* Sort newer threads before older ones. */
  if (al > 0 && bl > 0)
    {
      if (ad[0] > bd[0])
	return 1;
      else if (ad[0] < bd[0])
	return 0;
    }

  /* But sort within threads in ascending order of posting. */
  for (i = 1; i < al && i < bl; i++)
    {
      if (ad[i] > bd[i])
	return 0;
      else if (ad[i] < bd[i])
	return 1;
    }

  return (al < bl);
}

int
_int4_eq (ArrayType *a, ArrayType *b)
{
  const int al = ARR_DIMS(a)[0];
  const int bl = ARR_DIMS(b)[0];
  const int *const ad = (int *) ARR_DATA_PTR(a);
  const int *const bd = (int *) ARR_DATA_PTR(b);
  int i;

  if (al != bl)
    return 0;
  for (i = 0; i < al && i < bl; i++)
    if (ad[i] != bd[i])
      return 0;

  return 1;
}

int
_int4_ne (ArrayType *a, ArrayType *b)
{
  const int al = ARR_DIMS(a)[0];
  const int bl = ARR_DIMS(b)[0];
  const int *const ad = (int *) ARR_DATA_PTR(a);
  const int *const bd = (int *) ARR_DATA_PTR(b);
  int i;

  if (al != bl)
    return 1;
  for (i = 0; i < al && i < bl; i++)
    if (ad[i] != bd[i])
      return 1;

  return 0;
}
