/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

/* This whole thing needs to be redone, dammit. */

#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"

#define COOKIE_NAME "scope-session="

typedef struct auth_config_struct
  {
    char *auth_dir;
    char *auth_redirect;
    int auth_authoritative;
  }
auth_config_rec;

typedef struct auth_request_struct
  {
    char *username;
    char *workgroup;
    char *session;
  }
auth_request_rec;

static void *
create_auth_dir_config (pool *p, char *d)
{
  auth_config_rec *r = ap_pcalloc (p, sizeof (auth_config_rec));
  r->auth_dir = NULL;
  r->auth_redirect = NULL;
  r->auth_authoritative = 1;
  return r;
}

static const char *
set_auth_slot (cmd_parms *cmd, void *offset, char *f, char *t)
{
  if (t && strcmp (t, "TeamSCOPE"))
    return ap_pstrcat (cmd->pool, "Invalid auth file type: ", t, NULL);

  return ap_set_file_slot (cmd, offset, f);
}

static const command_rec auth_cmds[] = 
  {
    {
      "AuthTeamSCOPEDir",
      set_auth_slot,
      (void *) XtOffsetOf (auth_config_rec, auth_dir),
      OR_AUTHCFG,
      TAKE12,
      "trusted state directory for TeamSCOPE authentication",
    },
    {
      "AuthTeamSCOPERedirect",
      set_auth_slot,
      (void *) XtOffsetOf (auth_config_rec, auth_redirect),
      OR_AUTHCFG,
      TAKE12,
      "redirect URL for failed authentication",
    },
    {
      "AuthTeamSCOPEAuthoritative",
      ap_set_flag_slot,
      (void *) XtOffsetOf (auth_config_rec, auth_authoritative),
      OR_AUTHCFG,
      FLAG,
      "If set to `no' failed authentication will be passed to lower modules",
    },
    {
      NULL,
    },
  };

module MODULE_VAR_EXPORT auth_TeamSCOPE_module;

static int
authorize_user (request_rec *r, char *auth_dir, char *session_id)
{
  char *fn;
  configfile_t *f;
  char line[MAX_STRING_LEN];
  char *usr, *grp;

  fn = ap_palloc (r->pool, strlen (auth_dir) + 1 + strlen (session_id) + 1);
  sprintf (fn, "%s/%s", auth_dir, session_id);

  f = ap_pcfg_openfile (r->pool, fn);
  if (f == NULL)
    {
      ap_log_rerror (APLOG_MARK, APLOG_ERR, r,
		     "couldn't open trusted data file: \"%s\"", fn);
      return 0;
    }

  usr = grp = NULL;
  while (!ap_cfg_getline (line, MAX_STRING_LEN, f))
    {
      if (!strncmp (line, "user=", 5))
	usr = ap_pstrdup (r->pool, line + 5);
      else if (!strncmp (line, "workgroup=", 10))
	grp = ap_pstrdup (r->pool, line + 10);
    }

  if (usr && grp)
    {
      auth_request_rec *rec;

      rec = ap_palloc (r->pool, sizeof (auth_request_rec));
      rec->username = usr;
      rec->workgroup = grp;
      rec->session = session_id;

      ap_set_module_config (r->request_config, &auth_TeamSCOPE_module, rec);
      return 1;
    }
  else
    return 0;
}

/* I don't doubt that there is a better way to do this in Apache.
   Unfortunately, I haven't been able to find it. */
static char *
concat_url_query_quoted (char *d, const char *s)
{
  for (;;)
    {
      unsigned char ch = *s++;
      int quote = 0;
      
      switch (ch)
	{
	case '\0':
	  *d = '\0';
	  return d;
	  
	case ' ':
	  *d++ = '+';
	  continue;
	  
	case '%': case '<': case '>': case '#': case '"':
	case '{': case '}': case '|': case '[': case ']':
	case '^': case '`': case ';': case '/': case '?':
	case ':': case '=': case ',': case '$': case '@':
	case '\\': case '&':
	  quote = 1;
	  break;

	default:
	  quote = ch < 0x20 || ch >= 0x7f;
	  break;
	}

      if (!quote)
	*d++ = ch;
      else
	{
	  static const char hex[] = "0123456789ABCDEF";
	  *d++ = '%';
	  *d++ = hex[ch >> 4];
	  *d++ = hex[ch & 0xf];
	}
    }
}

static char *
get_requirements (request_rec *r)
{
  char *str;
  size_t len;

  const array_header *reqs_arr;
  require_line *reqs;
  int i;
  
  reqs_arr = ap_requires (r);
  if (reqs_arr == NULL)
    return OK;
  reqs = (require_line *) reqs_arr->elts;

  str = NULL;
  len = 0;
  for (i = 0; i < reqs_arr->nelts; i++)
    {
      char *requirement;
      if (!(reqs[i].method_mask & (1 << r->method_number)))
	continue;

      requirement = reqs[i].requirement;
      str = realloc (str, len + 3 * strlen (requirement) + 64);
      if (str == NULL)
	{
	  fprintf (stderr, "malloc failed in get_requirements()\n");
	  exit (1);
	}

      len += sprintf (str + len, "&need=");
      len = concat_url_query_quoted (str + len, requirement) - str;
    }

  return str ? str : "";
}
      
static int
auth_failure (request_rec *r)
{
  auth_config_rec *rec;

  rec = ap_get_module_config (r->per_dir_config, &auth_TeamSCOPE_module);
  if (rec && rec->auth_redirect)
    {
      char *requirements;
      char *url;
      char *redirect;
      char *p;

      url = ap_construct_url (r->pool, rec->auth_redirect, r);
      requirements = get_requirements (r);
      
      p = redirect = ap_palloc (r->pool,
				strlen (url) + strlen (r->uri) * 3
				+ strlen (requirements) + 64);
      p += sprintf (p, "%s/auth?redirect=", url);
      p = concat_url_query_quoted (p, r->uri);
      p += sprintf (p, "%s", requirements);

      free (requirements);
      
      ap_table_setn (r->headers_out, "Location", redirect);

      return REDIRECT;
    }
  else
    {
      ap_note_basic_auth_failure (r);

      return AUTH_REQUIRED;
    }
}

/* Clean up and free DATA allocated by make_session_id_pipe below. */
static void
cleanup_pipe (void *data)
{
  int *pipe = data;
  close (*pipe);
  free (data);
}

/* It's not safe to pass the session ID through the environment,
   because that can be seen by anyone with an account on the machine.
   So, we create a pipe and write the session ID to the pipe, and save
   the file descriptor for the pipe into the environment instead.  We
   also have to do some ugly screwing around with Apache to make sure
   that it doesn't close the pipe before it gets passed to the
   subprocess, but deal with it. */
static int
make_session_id_pipe (request_rec *r, const char *session_id)
{
  int pipefd[2];

  if (pipe (pipefd) == -1)
    {
      ap_log_rerror (APLOG_MARK, APLOG_ERR, r, "pipe() returned error");
      return 0;
    }

  {
    ssize_t amt;

    for (;;)
      {
	errno = 0;
	amt = write (pipefd[1], session_id, 41);
	if (amt == 41)
	  break;
      
	if (amt == -1 && errno == EINTR)
	  continue;

	ap_log_rerror (APLOG_MARK, APLOG_ERR, r,
		       "error writing to pipe or short write");
	close (pipefd[0]);
	close (pipefd[1]);
	return 0;
      }
  }

  close (pipefd[1]);
  
  {
    int *cleanup;

    cleanup = malloc (sizeof (int));
    if (cleanup == NULL)
      {
	ap_log_rerror (APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
		       "malloc() failed");
	close (pipefd[0]);
	return 0;
      }
    *cleanup = pipefd[0];

    ap_block_alarms ();
    ap_register_cleanup (r->pool, cleanup, cleanup_pipe, ap_null_cleanup);
    ap_unblock_alarms ();
  }

  ap_table_set (r->subprocess_env, "SCOPE_SESSION",
		ap_psprintf (r->pool, "%d", pipefd[0]));

  return 1;
}

/* Verify that the user supplied a session-id of the proper form and
   didn't slip in any of nasty characters like / or . */
static int
verify_session_id (request_rec *r, const char *session_id)
{
  if (strlen (session_id) != 41 || session_id[20] != '-')
    goto fail;

  {
    int i;

    for (i = 0; i < 41; i++)
      if (i != 20 && !isxdigit (session_id[i]))
	goto fail;
  }

  return 1;

 fail:
  ap_log_rerror (APLOG_MARK, APLOG_ERR, r, 
		 "session ID not of proper form: \"%s\"", session_id);
  return 0;
}

static char *
extract_session_id (request_rec *r)
{
  auth_request_rec *rec;
  const char *cookie;
  char *session;

  rec = ap_get_module_config (r->request_config, &auth_TeamSCOPE_module);
  if (rec && rec->session)
    return rec->session;

  /* This is a crock of shit.  But it's the only way I can figure out
     to keep Apache from passing the Cookie value into the
     environment.  However, it's nearly 2am at the moment, so this
     might just be lack of sleep talking.

     Actually a better way would be to reimplement bits of mod_cgi in
     mod_auth_scope.  I'm out of time though, and this seems to
     work. */
  cookie = ap_table_get (r->headers_in, "Cookie");  
  if (cookie == NULL)
    cookie = ap_table_get (r->headers_in, "Authorization");
  else
    {
      ap_table_setn (r->headers_in, "Authorization", cookie);
      ap_table_unset (r->headers_in, "Cookie");
    }
  if (cookie == NULL)
    return 0;

  session = strstr (cookie, COOKIE_NAME);
  if (session == NULL)
    return NULL;
  
  {
    char *end;
	  
    session += strlen (COOKIE_NAME);
    if (*session == '"')
      session++;
    session = ap_pstrdup (r->pool, session);

    end = strpbrk (session, "\";");
    if (end)
      *end = '\0';
  }

  if (!verify_session_id (r, session))
    return NULL;

  if (rec == NULL)
    {
      rec = ap_palloc (r->pool, sizeof (auth_request_rec));
      rec->username = NULL;
      rec->workgroup = NULL;
    }
  rec->session = session;

  ap_set_module_config (r->request_config, &auth_TeamSCOPE_module, rec);

  return session;
}

static int 
authenticate_TeamSCOPE_user (request_rec *r)
{
  auth_config_rec *auth;
  char *session;

  auth = ap_get_module_config (r->per_dir_config, &auth_TeamSCOPE_module);
  if (!auth->auth_dir)
    return DECLINED;

  session = extract_session_id (r);
  if (!session || !authorize_user (r, auth->auth_dir, session))
    {
      ap_log_rerror (APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
		     "authentication failure for \"%s\", "
		     "session %.20s-<censored>",
		     r->uri, session ? session : "<unknown>");
      return auth_failure (r);
    }

  return OK;
}

static int
check_user_access (request_rec *r)
{
  const char *usr, *grp;

  auth_config_rec *auth;
  const array_header *reqs_arr;
  require_line *reqs;
  int restricted;
  int i;

  auth = ap_get_module_config (r->per_dir_config, &auth_TeamSCOPE_module);
  reqs_arr = ap_requires (r);
  if (reqs_arr == NULL)
    return OK;
  reqs = (require_line *) reqs_arr->elts;

  {
    auth_request_rec *rec;

    rec = ap_get_module_config (r->request_config, &auth_TeamSCOPE_module);
    if (rec == NULL)
      {
	usr = NULL;
	grp = NULL;
      }
    else
      {
	usr = rec->username;
	grp = rec->workgroup;
      }
  }

  restricted = 0;
  for (i = 0; i < reqs_arr->nelts; i++)
    {
      const char *p;
      const char *w;

      if (!(reqs[i].method_mask & (1 << r->method_number)))
	continue;

      restricted = 1;
      p = reqs[i].requirement;
      w = ap_getword_white (r->pool, &p);

      if (!strcmp (w, "valid-user"))
	return OK;
      else if (!strcmp (w, "user"))
	{
	  while (p[0])
	    {
	      w = ap_getword_conf (r->pool, &p);
	      if (usr && !strcmp (usr, w))
		return OK;
	    }
	}
      else if (!strcmp (w, "group"))
	{
	  while (p[0])
	    {
	      w = ap_getword_conf (r->pool, &p);
	      if (grp && !strcmp (grp, w))
		return OK;
	    }
	}
      else if (auth->auth_authoritative)
	{
	  ap_log_rerror (APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
			 "access to \"%s\" failed: unknown require "
			 "directive: \"%s\"",
			 r->uri, reqs[i].requirement);
	}
    }

  if (!restricted)
    return OK;

  if (!auth->auth_authoritative)
    return DECLINED;

  ap_log_rerror (APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
		 "access to \"%s\" failed: user \"%s\", "
		 "workgroup \"%s\" not allowed access",
		 r->uri, usr ? usr : "<unknown>", grp ? grp : "<unknown>");

  return auth_failure (r);
}

static int
clean_environment (request_rec *r)
{
  const char *session = extract_session_id (r);
  if (!session)
    return DECLINED;

  if (!make_session_id_pipe (r, session))
    return auth_failure (r);

  return OK;
}

module MODULE_VAR_EXPORT auth_TeamSCOPE_module =
{
    STANDARD_MODULE_STUFF,
    NULL,			/* initializer */
    create_auth_dir_config,	/* dir config creater */
    NULL,			/* dir merger --- default is to override */
    NULL,			/* server config */
    NULL,			/* merge server config */
    auth_cmds,			/* command table */
    NULL,			/* handlers */
    NULL,			/* filename translation */
    authenticate_TeamSCOPE_user,/* check_user_id */
    check_user_access,		/* check auth */
    NULL,			/* check access */
    NULL,			/* type_checker */
    clean_environment,		/* fixups */
    NULL,			/* logger */
    NULL,			/* header parser */
    NULL,			/* child_init */
    NULL,			/* child_exit */
    NULL			/* post read-request */
};
